from setuptools import setup
from kz159_utils import version

setup(
    name="kz159_utils",
    version=version,
    packages=["kz159_utils"],
    url="https://github.com/kz159/simple_config",
    license="MIT",
    author="kz159",
    author_email="me@kz159.ru",
    description="",
)
